
-- GLOBALS: C_Reputation, _G, GetProfessions, GetProfessionInfo, GetNumFactions, GetFactionInfo, UnitSex, GetText, BreakUpLargeNumbers
-- GLOBALS:		GetFriendshipReputation, GetSelectedFaction, IsFactionInactive
-- GLOBALS: StatusTrackingBarManager, ReputationFrame, ReputationListScrollFrame, ReputationListScrollFrameScrollBar, FauxScrollFrame_Update, FauxScrollFrame_GetOffset
-- GLOBALS:		ReputationFrameStandingLabel, ReputationDetailFrame, ReputationDetailFactionName, ReputationDetailFactionDescription, ReputationFrame_SetRowType
-- GLOBALS:		ReputationDetailAtWarCheckBox, ReputationDetailAtWarCheckBoxText, ReputationDetailInactiveCheckBox, ReputationDetailInactiveCheckBoxText
-- GLOBALS:		ReputationDetailMainScreenCheckBox
-- GLOBALS: NUM_FACTIONS_DISPLAYED, REPUTATIONFRAME_FACTIONHEIGHT, MAX_REPUTATION_REACTION, HIGHLIGHT_FONT_COLOR_CODE, REPUTATION_PROGRESS_FORMAT, FONT_COLOR_CODE_CLOSE
-- GLOBALS:		RED_FONT_COLOR, GRAY_FONT_COLOR, FACTION_BAR_COLORS
-- GLOBALS: MFAC_Data

local pairs, format, tinsert = pairs, format, tinsert

local ADDON_NAME, MFAC = ... -- Pulls back the Addon-Local Variables and store them locally.

function MFAC.IsFactionParagon(p_faction_id)
	return C_Reputation.IsFactionParagon(p_faction_id);
end

function MFAC.GetFactionParagonInfo(p_faction_id)
	local currentValue, threshold, rewardQuestID, hasRewardPending, tooLowLevelForParagon = C_Reputation.GetFactionParagonInfo(p_faction_id);
	return currentValue, threshold, rewardQuestID, hasRewardPending, tooLowLevelForParagon;
end

function MFAC.ShowReputationBarHighlight(p_rep_bar)

	_G["ReputationBar" .. p_rep_bar .. "ReputationBarHighlight1"]:Show();
	_G["ReputationBar" .. p_rep_bar .. "ReputationBarHighlight2"]:Show();

end

function MFAC.HideReputationBarHighlight(p_rep_bar)

	_G["ReputationBar" .. p_rep_bar .. "ReputationBarHighlight1"]:Hide();
	_G["ReputationBar" .. p_rep_bar .. "ReputationBarHighlight2"]:Hide();

end


function MFAC.ExtractSkills()
	local _

	local skill_line1, skill_line2
	local prof1, prof2, archaeology, fishing, cooking = GetProfessions();
	if (prof1) then
		_, _, _, _, _, _, skill_line1 = GetProfessionInfo(prof1);
	end
	if (prof2) then
		_, _, _, _, _, _, skill_line2 = GetProfessionInfo(prof2);
	end

	local limit = MFAC.LimitType;
	MFAC.SkillTrackBits = 0;

	for k,v in pairs(MFAC.SkillLine) do
		if(skill_line1 == v) or (skill_line2 == v) then
			--print("You have ", k, limit[k]);
			MFAC.SkillTrackBits = MFAC.SkillTrackBits + limit[k];
		end
	end

	if (archaeology) then
		MFAC.SkillTrackBits = MFAC.SkillTrackBits + limit.Archaeology;
	end
	if (fishing) then
		MFAC.SkillTrackBits = MFAC.SkillTrackBits + limit.Fishing;
	end
	if (cooking) then
		MFAC.SkillTrackBits = MFAC.SkillTrackBits + limit.Cooking;
	end


end

function MFAC.StatusTrackingBarManager_UpdateBarsShown()

	StatusTrackingBarManager:UpdateBarsShown();

end

function MFAC.GetAllFactionInfo(p_faction_id)
	assert(p_faction_id == nil or p_faction_id >= 0)

	local name, description, standingID, barMin, barMax, barValue, atWarWith, canToggleAtWar, isHeader,
			isCollapsed, hasRep, isWatched, isChild, factionID, hasBonusRepGain = GetFactionInfoByID(p_faction_id)

	local faction_tmp = {name = name, description = description, standing_id = standingID, bar_min = barMin, bar_max = barMax, bar_value = barValue, at_war = atWarWith,
			can_toggle_at_war = canToggleAtWar, is_header = isHeader, is_collapsed = isCollapsed, has_rep = hasRep, is_watched = isWatched, is_child = isChild,
			faction_id = factionID, has_bonus_rep_gain = hasBonusRepGain};

	local currentValue, threshold, rewardQuestID, hasRewardPending, tooLowLevelForParagon = C_Reputation.GetFactionParagonInfo(p_faction_id)
	local paragon_tmp = {current_value = currentValue, threshold = threshold, reward_quest_id = rewardQuestID, has_reward_pending = hasRewardPending, too_low_level = tooLowLevelForParagon};

	local friendID, friendRep, friendMaxRep, friendName, friendText, friendTexture, friendTextLevel, friendThreshold, nextFriendThreshold = GetFriendshipReputation(factionID);
	local friend_tmp = {friend_id = friendID, current_rep = friendRep, max_rep = friendMaxRep, name = friendName, description = friendText, texture = friendTexture,
		standing_text = friendTextLevel, threshold = friendThreshold, next_threshold = nextFriendThreshold}

	if (not MFAC.is_table_empty(paragon_tmp) and MFAC_Data.ShowParagon and faction_tmp.standing_id == 8) then
		faction_tmp.standing_id = 9
	end

	return { faction = faction_tmp, paragon = paragon_tmp, friend = friend_tmp}
end

function MFAC.GetHeaderFactionInfo(p_faction_idx)
	assert(p_faction_idx > 0)

	local name, description, standingID, barMin, barMax, barValue, atWarWith, canToggleAtWar, isHeader,
			isCollapsed, hasRep, isWatched, isChild, factionID, hasBonusRepGain = GetFactionInfo(p_faction_idx)

	local faction_tmp = {name = name, description = description, standing_id = standingID, bar_min = barMin, bar_max = barMax, bar_value = barValue, at_war = atWarWith,
			can_toggle_at_war = canToggleAtWar, is_header = isHeader, is_collapsed = isCollapsed, has_rep = hasRep, is_watched = isWatched, is_child = isChild,
			faction_id = factionID, has_bonus_rep_gain = hasBonusRepGain};

	local paragon_tmp = {};	--ToDo

	local friend_tmp = {};	--ToDo

	-- So far Paragon Headers aren't a thing, but who knows?
	if (not MFAC.is_table_empty(paragon_tmp) and MFAC_Data.ShowParagon and faction_tmp.standing_id == 8) then
		faction_tmp.standing_id = 9
	end

	return { faction = faction_tmp, paragon = paragon_tmp, friend = friend_tmp}
end

function MFAC.GetStandardFactionData()

	local faction_data = {}
	local num_factions = GetNumFactions()

	for i = 1, num_factions do
		local faction_id = select(14, GetFactionInfo(i))
		if (faction_id == nil) then
			faction_data[i] = MFAC.GetHeaderFactionInfo(i)
		else
			faction_data[i] = MFAC.GetAllFactionInfo(faction_id)
		end
	end

	return num_factions, faction_data
end

function MFAC.GetFactionDataByStanding()

	local faction_data = {}
	local standing_bucket = {}
	local num_factions = GetNumFactions()
	local sex = UnitSex("player")

	for i = 1, num_factions do
		local faction_id = select(14, GetFactionInfo(i))
		--print(i, GetFactionInfo(i), faction_id)
		local tmp = faction_id and MFAC.GetAllFactionInfo(faction_id)
		if (not (faction_id == nil or tmp.faction.is_header)) then	--Skip headers in this sort mode. The Other & Inactive headers have a faction_id of nil
			local standing_id = tmp.faction.standing_id
			if (tmp.friend.friend_id and not tmp.friend.next_threshold) then	--it's a friend with max rep, so bucket it with Exalted
				standing_id = MAX_REPUTATION_REACTION
			end
			if (not MFAC.CollapsedHeader[standing_id]) then
				standing_bucket[standing_id] = standing_bucket[standing_id] or {}
				tinsert(standing_bucket[standing_id], tmp)
			end
		end
	end

	local num_sorted_factions = 0
	for i = 9, 1, -1 do
		if (standing_bucket[i] or MFAC.CollapsedHeader[i]) then
			local header = { faction = {standing_id = i, is_header = true, faction_id = 0, is_collapsed = MFAC.CollapsedHeader[i] }, paragon = {}, friend = {}}
			local name = MFAC.get_faction_standing(header, sex)
			header.faction.name = name
			num_sorted_factions = num_sorted_factions + 1
			tinsert(faction_data, header)
			-- Add factions below Header
			standing_bucket[i] = standing_bucket[i] or {}
			for _k, v in ipairs(standing_bucket[i]) do
				num_sorted_factions = num_sorted_factions + 1
				v.faction.is_child = false
				tinsert(faction_data, v)
			end
		end
	end


	return num_sorted_factions, faction_data
end

function MFAC.ExpandOrCollapseButton_OnClick_Original(p_self, _p_button, _p_down)
	if (p_self:GetParent().isCollapsed) then
		ExpandFactionHeader(p_self:GetParent().index);
	  else
		CollapseFactionHeader(p_self:GetParent().index);
	  end
end

function MFAC.ExpandOrCollapseButton_OnClick_ByStanding(p_self, _p_button, _p_down)
	local standing_id = p_self:GetParent().standing_id

	MFAC.CollapsedHeader[standing_id] = not MFAC.CollapsedHeader[standing_id]
	MFAC.ReputationFrame_Update()
end

function MFAC.ReputationFrame_Update()
	if (ReputationFrame.paragonFramesPool) then
		ReputationFrame.paragonFramesPool:ReleaseAll();
	end

	local num_factions, faction_data
	if MFAC_Data.RepGroupBy == MFAC.RepGroupByOptions.Standing then
		num_factions, faction_data = MFAC.GetFactionDataByStanding()
	else
		num_factions, faction_data = MFAC.GetStandardFactionData()
	end

	-- Update scroll frame
	if ( not FauxScrollFrame_Update(ReputationListScrollFrame, num_factions, NUM_FACTIONS_DISPLAYED, REPUTATIONFRAME_FACTIONHEIGHT ) ) then
		ReputationListScrollFrameScrollBar:SetValue(0);
	end
	local factionOffset = FauxScrollFrame_GetOffset(ReputationListScrollFrame);

	-- Set "Standing" title on Reputation frame
	if (MFAC_Data.ShowMissing) then
		ReputationFrameStandingLabel:SetText(MFAC.original_standing_text .. " " .. FIZ_TXT.missing)
	else
		ReputationFrameStandingLabel:SetText(MFAC.original_standing_text)
	end

	if (MFAC_Data.ShowPreviewRep) then
		FIZ_ParseBagContent()
	end

	local gender = UnitSex("player");

	for i = 1, NUM_FACTIONS_DISPLAYED, 1 do
		local factionIndex = factionOffset + i;
		local factionRow = _G["ReputationBar"..i];
		if ( factionIndex > num_factions ) then
			factionRow:Hide();
		else
			local factionBar = _G["ReputationBar"..i.."ReputationBar"];
			local factionTitle = _G["ReputationBar"..i.."FactionName"];
			local factionButton = _G["ReputationBar"..i.."ExpandOrCollapseButton"];
			local factionStanding = _G["ReputationBar"..i.."ReputationBarFactionStanding"];
			local factionBarPreview = _G["FIZ_StatusBar"..i];
			local this_faction = faction_data[factionIndex].faction
			local friend = faction_data[factionIndex].friend

			factionTitle:SetText(this_faction.name);

			if  MFAC_Data.RepGroupBy == MFAC.RepGroupByOptions.Standing then
				factionButton:SetScript("OnClick", MFAC.ExpandOrCollapseButton_OnClick_ByStanding)
			else
				factionButton:SetScript("OnClick", MFAC.ExpandOrCollapseButton_OnClick_Original)
			end

			factionRow.isCollapsed = this_faction.is_collapsed;
			if (this_faction.is_collapsed ) then
				factionButton:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up");
			else
				factionButton:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up");
			end

			factionRow.index = factionIndex;
			factionRow.faction_id = this_faction.faction_id;
			factionRow.standing_id = this_faction.standing_id;

			local barMax, barValue = 0, 0
			-- We only need to bother with calculating the Rep values if this Faction *has* a rep value
			if (this_faction.is_header and this_faction.has_rep) or (not this_faction.is_header) then
				barMax, barValue = MFAC.get_bar_values_for_faction(faction_data[factionIndex])
			end

			local factionStandingtext, colorIndex = MFAC.get_faction_standing(faction_data[factionIndex], gender)

			MFAC.update_paragon_frame(faction_data[factionIndex], factionRow)

			local isCapped = (this_faction.standing_id == MAX_REPUTATION_REACTION) or (friend.friend_id and not friend.next_threshold)
			local isCappedFriendship = not friend.next_threshold

			local missing_text = ""
			if (MFAC_Data.ShowMissing) then
				factionRow.rolloverText = nil;
				missing_text = " (" .. BreakUpLargeNumbers(barMax - barValue) .. ")";
			elseif isCapped then
				factionRow.rolloverText = nil;
			else
				factionRow.rolloverText = HIGHLIGHT_FONT_COLOR_CODE .. " " .. format(REPUTATION_PROGRESS_FORMAT, BreakUpLargeNumbers(barValue), BreakUpLargeNumbers(barMax)) .. FONT_COLOR_CODE_CLOSE;
			end

			if ( isCappedFriendship ) then
				factionRow.tooltip = nil;
			else
				factionRow.tooltip = HIGHLIGHT_FONT_COLOR_CODE.." "..barValue.." / "..barMax..FONT_COLOR_CODE_CLOSE;
			end

			factionRow.standingText = factionStandingtext .. missing_text;
			factionStanding:SetText(factionRow.standingText);
			factionBar:SetFillStyle("STANDARD_NO_RANGE_FILL");
			factionBar:SetMinMaxValues(0, barMax);
			factionBar:SetValue(barValue);
			factionBar:SetFrameLevel(5)
			local color = FACTION_BAR_COLORS[colorIndex];
			factionBar:SetStatusBarColor(color.r, color.g, color.b);
			factionBar.BonusIcon:SetShown(this_faction.has_bonus_rep_gain);

			local previewValue = 0
			if (factionBarPreview and MFAC_Data.ShowPreviewRep) then
				previewValue = FIZ_GetReadyReputation(faction_data[factionIndex])
			end
--if (i < 16) then print(previewValue, this_faction.standing_id); end
			if ((previewValue > 0) and (this_faction.standing_id ~=8)) then
				factionBarPreview:Show()
				factionBarPreview:SetMinMaxValues(0, barMax)
				previewValue = previewValue + barValue
				if (previewValue > barMax) then previewValue = barMax end
				factionBarPreview:SetValue(previewValue)
				factionBarPreview:SetID(factionIndex)

				factionBarPreview:SetStatusBarTexture("Interface\\Addons\\" .. ADDON_NAME .. "\\Textures\\UI-Character-Skills-Bar.blp")
				factionBarPreview:SetStatusBarColor(1.0, 1.0, 1.0, 0.8)
			else
				factionBarPreview:Hide()
			end

			FIZ_ReputationFrame_SetRowType(factionRow, this_faction.is_child, this_faction.is_header, this_faction.has_rep);
			factionRow:Show();
			if ( this_faction.at_war ) then
				_G["ReputationBar"..i.."ReputationBarAtWarHighlight1"]:Show();
				_G["ReputationBar"..i.."ReputationBarAtWarHighlight2"]:Show();
			else
				_G["ReputationBar"..i.."ReputationBarAtWarHighlight1"]:Hide();
				_G["ReputationBar"..i.."ReputationBarAtWarHighlight2"]:Hide();
			end
			if ( factionIndex == GetSelectedFaction() ) then
				if ( ReputationDetailFrame:IsShown() ) then
					MFAC.UpdateOriginalReputationDetailFrame(this_faction, factionIndex, i)

					ReputationDetailFrame:SetHeight(203);
				end
				if ( MFAC.UI.ReputationDetailFrame:IsVisible() ) then
					local toExalted = 0
					if (this_faction.standing_id < MAX_REPUTATION_REACTION) then
						toExalted = MFAC.ToExalted[this_faction.standing_id] + barMax - barValue;
					end
					FIZ_Rep_Detail_Frame(factionIndex, faction_data[factionIndex], color, barValue, barMax, toExalted, factionStandingtext)
					MFAC.ShowReputationBarHighlight(i)
				end
			else
				MFAC.HideReputationBarHighlight(i)
			end

		end
	end

	if ( GetSelectedFaction() == 0 ) then
		ReputationDetailFrame:Hide();
		MFAC.UI.ReputationDetailFrame:Hide();
	end

end

function MFAC.update_paragon_frame(p_faction_info, p_faction_row)
	local factionID = p_faction_info.faction.faction_id;
	local paragon = p_faction_info.paragon

	if (paragon.current_value ~= nil) then
		local paragonFrame = ReputationFrame.paragonFramesPool:Acquire();
		paragonFrame.factionID = factionID;
		paragonFrame:SetPoint("RIGHT", p_faction_row, 11, 0);
		C_Reputation.RequestFactionParagonPreloadRewardData(factionID);
		paragonFrame.Glow:SetShown(not paragon.too_low_level and paragon.has_reward_pending);
		paragonFrame.Check:SetShown(not paragon.too_low_level and paragon.has_reward_pending);

		if(not paragon.has_reward_pending and MFAC_Data.FadeParagonBags) then
			paragonFrame:SetAlpha(0.5)
		else
			paragonFrame:SetAlpha(1)
		end

		paragonFrame:Show();
	end

end
